Shader "Nature/Terrain/Standard" {
	Properties{
		// set by terrain engine
		[HideInInspector] _Control("Control (RGBA)", 2D) = "red" {}
	[HideInInspector] _Splat3("Layer 3 (A)", 2D) = "white" {}
	[HideInInspector] _Splat2("Layer 2 (B)", 2D) = "white" {}
	[HideInInspector] _Splat1("Layer 1 (G)", 2D) = "white" {}
	[HideInInspector] _Splat0("Layer 0 (R)", 2D) = "white" {}
	[HideInInspector] _Normal3("Normal 3 (A)", 2D) = "bump" {}
	[HideInInspector] _Normal2("Normal 2 (B)", 2D) = "bump" {}
	[HideInInspector] _Normal1("Normal 1 (G)", 2D) = "bump" {}
	[HideInInspector] _Normal0("Normal 0 (R)", 2D) = "bump" {}
	[HideInInspector][Gamma] _Metallic0("Metallic 0", Range(0.0, 1.0)) = 0.0
		[HideInInspector][Gamma] _Metallic1("Metallic 1", Range(0.0, 1.0)) = 0.0
		[HideInInspector][Gamma] _Metallic2("Metallic 2", Range(0.0, 1.0)) = 0.0
		[HideInInspector][Gamma] _Metallic3("Metallic 3", Range(0.0, 1.0)) = 0.0
		[HideInInspector] _Smoothness0("Smoothness 0", Range(0.0, 1.0)) = 1.0
		[HideInInspector] _Smoothness1("Smoothness 1", Range(0.0, 1.0)) = 1.0
		[HideInInspector] _Smoothness2("Smoothness 2", Range(0.0, 1.0)) = 1.0
		[HideInInspector] _Smoothness3("Smoothness 3", Range(0.0, 1.0)) = 1.0

		// used in fallback on old cards & base map
		[HideInInspector] _MainTex("BaseMap (RGB)", 2D) = "white" {}
	[HideInInspector] _Color("Main Color", Color) = (1,1,1,1)
	}

		SubShader{
				Tags{
				"Queue" = "Geometry-100"
				"RenderType" = "Opaque"
				"PerformanceChecks" = "False"
				}

			Pass
			{
				Name "FORWARD"
				Tags{ "LightMode" = "ForwardBase" "PassFlags" = "OnlyDirectional" } // NOTE: "OnlyDirectional" prevents Unity from baking dynamic lights into SH terms at runtime

				CGPROGRAM
		#pragma target 5.0
		#pragma only_renderers d3d11
		#pragma exclude_renderers gles
		#pragma vertex MainVs
		#pragma fragment MainPs
		#pragma shader_feature S_RECEIVE_SHADOWS
		#pragma multi_compile _ D_VALVE_SHADOWING_POINT_LIGHTS
		#pragma shader_feature S_OVERRIDE_LIGHTMAP
		#pragma multi_compile LIGHTMAP_OFF LIGHTMAP_ON
		#pragma multi_compile DIRLIGHTMAP_OFF DIRLIGHTMAP_COMBINED DIRLIGHTMAP_SEPARATE
		#pragma multi_compile DYNAMICLIGHTMAP_OFF DYNAMICLIGHTMAP_ON
		// Includes -------------------------------------------------------------------------------------------------------------------------------------------------
#include "UnityCG.cginc"
#include "UnityLightingCommon.cginc"
#include "UnityStandardUtils.cginc"
#include "UnityStandardInput.cginc"

#define S_RECEIVE_SHADOWS 1
#define D_VALVE_SHADOWING_POINT_LIGHTS 1
#include "Lighting.cginc"

#pragma multi_compile __ _TERRAIN_NORMAL_MAP
#define TERRAIN_STANDARD_SHADER
#define TERRAIN_SURFACE_OUTPUT SurfaceOutputStandard
#include "TerrainSplatmapCommon.cginc"
#include "vr_utils.cginc"
#include "vr_lighting.cginc"
#include "vr_matrix_palette_skinning.cginc"
#include "vr_fog.cginc"

#define LIGHTMAP_ON 1
#define DYNAMICLIGHTMAP_ON 1
#define DYNAMICLIGHTMAP_OFF 0
#define DIRLIGHTMAP_COMBINED 1
#define S_OVERRIDE_LIGHTMAP 0

	half _Metallic0;
	half _Metallic1;
	half _Metallic2;
	half _Metallic3;

	half _Smoothness0;
	half _Smoothness1;
	half _Smoothness2;
	half _Smoothness3;

	// Structs --------------------------------------------------------------------------------------------------------------------------------------------------
	struct VS_INPUT
{
	float4 vPositionOs : POSITION;
	float3 vNormalOs : NORMAL;
	float2 vTexCoord0 : TEXCOORD0;
	float2 vTexCoord1 : TEXCOORD1;
#if ( DYNAMICLIGHTMAP_ON || UNITY_PASS_META )
		float2 vTexCoord2 : TEXCOORD2;
#endif
	};

	struct PS_INPUT
	{
		float4 vPositionPs : SV_Position;
		float3 vPositionWs : TEXCOORD0;
		float3 vNormalWs : TEXCOORD1;
		float2 vTextureCoords : TEXCOORD2;
		float4 vLightmapUV : TEXCOORD3;
		float2 vFogCoords : TEXCOORD4;
		float4 pack0 : TEXCOORD5; // _Splat0 _Splat1
		float4 pack1 : TEXCOORD6; // _Splat2 _Splat3
		float2 custompack0 : TEXCOORD7; // tc_Control
	};

	float g_flValveGlobalVertexScale = 1.0; // Used to "hide" all valve materials for debugging

	// World-aligned texture
	float3 g_vWorldAlignedTextureSize = float3(1.0, 1.0, 1.0);
	float3 g_vWorldAlignedNormalTangentU = float3(-1.0, 0.0, 0.0);
	float3 g_vWorldAlignedNormalTangentV = float3(0.0, 0.0, 1.0);
	float3 g_vWorldAlignedTexturePosition = float3(0.0, 0.0, 0.0);

	float4 _Splat0_ST;
	float4 _Splat1_ST;
	float4 _Splat2_ST;
	float4 _Splat3_ST;
	// MainVs ---------------------------------------------------------------------------------------------------------------------------------------------------
	PS_INPUT MainVs(appdata_full i)
	{
		PS_INPUT o = (PS_INPUT)0;

		UNITY_SETUP_INSTANCE_ID(i);
		UNITY_TRANSFER_INSTANCE_ID(v, o);
		Input customInputData;
		SplatmapVert(i, customInputData);
		o.custompack0.xy = customInputData.tc_Control;

		float3 vPositionWs = mul(unity_ObjectToWorld, i.vertex).xyz;
		o.vPositionWs.xyz = vPositionWs.xyz;
		o.vPositionPs.xyzw = mul(UNITY_MATRIX_MVP, i.vertex.xyzw);

		// Normal
		float3 vNormalWs = UnityObjectToWorldNormal(i.normal);
		o.vNormalWs.xyz = vNormalWs.xyz;
#if ( LIGHTMAP_ON )
		o.vLightmapUV.xy = i.texcoord1.xy * unity_LightmapST.xy + unity_LightmapST.zw;
#endif

#if ( DYNAMICLIGHTMAP_ON )
		o.vLightmapUV.zw = i.texcoord2.xy * unity_DynamicLightmapST.xy + unity_DynamicLightmapST.zw;
#endif

		o.vFogCoords.xy = CalculateFogCoords(vPositionWs.xyz);

		o.pack0.xy = TRANSFORM_TEX(i.texcoord, _Splat0);
		o.pack0.zw = TRANSFORM_TEX(i.texcoord, _Splat1);
		o.pack1.xy = TRANSFORM_TEX(i.texcoord, _Splat2);
		o.pack1.zw = TRANSFORM_TEX(i.texcoord, _Splat3);
		return o;
	}

	// MainPs ---------------------------------------------------------------------------------------------------------------------------------------------------
	struct PS_OUTPUT
	{
		float4 vColor : SV_Target0;
	};

	PS_OUTPUT MainPs(PS_INPUT i)
	{
		PS_OUTPUT po = (PS_OUTPUT)0;

		Input surfIN;
		UNITY_INITIALIZE_OUTPUT(Input, surfIN);
		surfIN.uv_Splat0.x = 1.0;
		surfIN.uv_Splat1.x = 1.0;
		surfIN.uv_Splat2.x = 1.0;
		surfIN.uv_Splat3.x = 1.0;
		surfIN.tc_Control.x = 1.0;
		surfIN.uv_Splat0 = i.pack0.xy;
		surfIN.uv_Splat1 = i.pack0.zw;
		surfIN.uv_Splat2 = i.pack1.xy;
		surfIN.uv_Splat3 = i.pack1.zw;
		surfIN.tc_Control = i.custompack0.xy;

		half4 splat_control;
		half weight;
		fixed4 mixedDiffuse;
		half4 defaultSmoothness = half4(_Smoothness0, _Smoothness1, _Smoothness2, _Smoothness3);
		SplatmapMix(surfIN, defaultSmoothness, splat_control, weight, mixedDiffuse, i.vNormalWs);
		//mixedDiffuse.rgb;weight;mixedDiffuse.a;dot(splat_control, half4(_Metallic0, _Metallic1, _Metallic2, _Metallic3));

		float3 vAlbedo = mixedDiffuse.rgb;

		float3 vTangentUWs = float3(1.0, 0.0, 0.0);
		float3 vTangentVWs = float3(0.0, 1.0, 0.0);
		float3 vGeometricNormalWs = float3(0.0, 0.0, 1.0);
		i.vNormalWs.xyz = normalize(i.vNormalWs.xyz);
		vGeometricNormalWs.xyz = i.vNormalWs.xyz;
		float3 vNormalWs = vGeometricNormalWs.xyz;
		float3 vNormalTs = float3(0.0, 0.0, 1.0);

		// Roughness //
		float2 vRoughness = float2(0.6, 0.6);
		// Reflectance and gloss
		float3 vReflectance = float3(0.0, 0.0, 0.0);
		float flGloss = 0.0;
		vRoughness.xy = (1.0 - flGloss).xx;

		LightingTerms_t lightingTerms;
		lightingTerms.vDiffuse.rgb = float3(1.0, 1.0, 1.0);
		lightingTerms.vSpecular.rgb = float3(0.0, 0.0, 0.0);
		lightingTerms.vIndirectDiffuse.rgb = float3(0.0, 0.0, 0.0);
		lightingTerms.vIndirectSpecular.rgb = float3(0.0, 0.0, 0.0);
		lightingTerms.vTransmissiveSunlight.rgb = float3(0.0, 0.0, 0.0);

		float flFresnelExponent = 5.0;
		float flMetalness = 0.2f;

		float4 vLightmapUV = float4(0.0, 0.0, 0.0, 0.0);
#if (LIGHTMAP_ON || DYNAMICLIGHTMAP_ON )
		vLightmapUV.xy = i.vLightmapUV.xy;
	#if ( DYNAMICLIGHTMAP_ON )
		vLightmapUV.zw = i.vLightmapUV.zw;
	#endif
		// Compute lighting
	lightingTerms = ComputeLighting(i.vPositionWs.xyz, vNormalWs.xyz, vTangentUWs.xyz, vTangentVWs.xyz, vRoughness.xy, vReflectance.rgb, flFresnelExponent, vLightmapUV.xyzw);

#if ( S_OCCLUSION )		
	float flOcclusion = tex2D(_OcclusionMap, i.vTextureCoords.xy).g;
	lightingTerms.vDiffuse.rgb *= LerpOneTo(flOcclusion, _OcclusionStrength * _OcclusionStrengthDirectDiffuse);
	lightingTerms.vSpecular.rgb *= LerpOneTo(flOcclusion, _OcclusionStrength * _OcclusionStrengthDirectSpecular);
	lightingTerms.vIndirectDiffuse.rgb *= LerpOneTo(flOcclusion, _OcclusionStrength * _OcclusionStrengthIndirectDiffuse);
	lightingTerms.vIndirectSpecular.rgb *= LerpOneTo(flOcclusion, _OcclusionStrength * _OcclusionStrengthIndirectSpecular);
#endif
#endif

	// Diffuse
		po.vColor.rgb = (lightingTerms.vDiffuse.rgb + lightingTerms.vIndirectDiffuse.rgb) * vAlbedo.rgb;

		po.vColor.rgb += lightingTerms.vIndirectSpecular.rgb; // Indirect specular applies its own fresnel in the forward lighting header file

		//po.vColor.rgb = lightingTerms.vDiffuse.rgb;
		po.vColor.a = mixedDiffuse.a;

		// Emission - Unity just adds the emissive term at the end instead of adding it to the diffuse lighting term. Artists may want both options.
		//float3 vEmission = Emission(i.vTextureCoords.xy);
		//po.vColor.rgb += vEmission.rgb;
		//o.vColor.rgb = ApplyFog(o.vColor.rgb, i.vFogCoords.xy, _FogMultiplier);
		// Dither to fix banding artifacts
		//po.vColor.rgb = ScreenSpaceDither(i.vPositionPs.xy);

		return po;
	}
	ENDCG
	}

	}
		Fallback "Nature/Terrain/Diffuse"
}
